import chalk from 'chalk'
import fs from 'fs'
import { fileURLToPath } from 'url'
import Listr from 'listr'
import ncp from 'ncp'
import path from 'path'
import { promisify } from 'util'
import { createSpinner } from 'nanospinner'
import { download } from 'obtain-git-repo'

const access = promisify(fs.access)
const copy = promisify(ncp)
const writeFileSync = promisify(fs.writeFileSync)

const gitRepositoryList = [
  {
    name: 'gx-design-pro',
    git: 'https://gitee.com/gx12358/vue-antd-admin.git#v2'
  }
]

async function copyTemplateFiles(templateDir, root) {
  return copy(templateDir, root, {
    clobber: false
  })
}

function writeFile(path, content) {
  if (Object.keys(content).length > 0) {
    writeFileSync(path, content)
  }
}

function editPackageJson(root, projectName) {
  let pkg = {}

  if (fs.existsSync(path.join(root, `package.json`))) {
    pkg = JSON.parse(
      fs.readFileSync(path.join(root, `package.json`), 'utf-8')
    )

    pkg.name = projectName
  }

  writeFile(path.join(root, 'package.json'), JSON.stringify(pkg, null, 2))
}

function doneLog(cwd, root) {
  if (root !== cwd) {
    console.log(chalk.blueBright(`cd ${path.relative(cwd, root)}`))
  }
  console.log(chalk.blueBright(`pnpm(yarn、npm) install`))
  console.log(chalk.blueBright(`pnpm(yarn、npm) run dev`))
  console.log()
}

export async function createProject(options) {
  const cwd = process.cwd()

  const root = path.join(cwd, options.projectName)

  const getProjectName = () =>
    options.projectName === '.' ? path.basename(path.resolve()) : options.projectName

  const downloadGit = gitRepositoryList.find(el => el.name === options.template)

  const templateDir = path.resolve(
    fileURLToPath(import.meta.url),
    '../..',
    `template-${options.template}`
  )

  try {
    await access(templateDir, fs.constants.R_OK)
  } catch (err) {
    if (!downloadGit) {
      console.log(chalk.redBright('模版目录不存在'))
      process.exit(1)
    }
  }

  //目录是否已经存在
  const dirIsExists = fs.existsSync(root)

  if (dirIsExists) {
    console.log(chalk.redBright('目录已经存在'))
    process.exit(1)
  }

  if (downloadGit) {
    const spinner = createSpinner(`开始下载...${downloadGit.git}`).start()
    // 下载git代码
    download(`direct:${downloadGit.git}`, root, { clone: true }, async function(err) {
      if (err) {
        console.log(err)
        spinner.error({ text: '下载失败' })
      } else {
        await editPackageJson(root, options.projectName || getProjectName())
        spinner.success({
          text: '项目创建成功，请依次执行以下命令'
        })
        doneLog(cwd, root)
      }
    })
  } else {
    let pkg = {}

    if (fs.existsSync(path.join(templateDir, `package.json`))) {
      pkg = JSON.parse(
        fs.readFileSync(path.join(templateDir, `package.json`), 'utf-8')
      )

      pkg.name = options.projectName || getProjectName()
    }
    const tasks = new Listr(
      [
        {
          title: '项目创建成功，请依次执行以下命令',
          task: () => copyTemplateFiles(templateDir, root)
        }
      ],
      {
        exitOnError: false
      }
    )

    await tasks.run()
    await editPackageJson(root)
    doneLog(cwd, root)
  }
  return true
}
