import type { ExtractPropTypes } from 'vue'
import { computed, defineComponent, unref } from 'vue'
import { useRouter } from 'vue-router'
import { useWindowSize } from '@vueuse/core'
import { default as ResizeObserver } from 'ant-design-vue/es/vc-resize-observer'
import Logo from '@/assets/logo.png'
import { globalHeaderProps } from './props'
import type { SiderMenuProps } from '../SiderMenu/SiderMenu'
import { defaultRenderCollapsedButton } from '../SiderMenu/SiderMenu'
import { siderMenuProps } from '../SiderMenu/props'
import BaseMenu from '../SiderMenu/BaseMenu'
import DeFaultRightContent from '../RightContent'
import LogoContent, { defaultRenderLogo } from '../LogoContent'
import { useRouteContext } from '../../RouteContext'

export const defaultHeaderProps = { ...siderMenuProps, ...globalHeaderProps }

export type DefaultHeaderProps = Partial<ExtractPropTypes<typeof defaultHeaderProps>> &
  Partial<SiderMenuProps>

const renderLogo = (
  menuHeaderRender: SiderMenuProps['menuHeaderRender'],
  logoDom: CustomRender
) => {
  if (menuHeaderRender === false) {
    return null
  }
  if (menuHeaderRender) {
    return menuHeaderRender(logoDom, null)
  }
  return logoDom
}

const RightContent = defineComponent({
  props: defaultHeaderProps,
  setup(props) {
    const rightSize = ref<number | string>('auto')

    const handleResize = ({ width }: { width: number }) => {
      rightSize.value = width
    }

    return () => (
      <div
        style={{
          minWidth: typeof rightSize.value === 'string' ? rightSize.value : `${rightSize.value}px`
        }}
      >
        <div
          style={{
            paddingRight: '8px'
          }}
        >
          <ResizeObserver onResize={handleResize}>
            {props.rightContentRender && typeof props.rightContentRender === 'function' ? (
              <div>
                {props.rightContentRender({
                  ...props
                })}
              </div>
            ) : (
              <DeFaultRightContent theme={props.theme} extra={props.extraRightDropdownRender} />
            )}
          </ResizeObserver>
        </div>
      </div>
    )
  }
})

export const DefaultHeader = defineComponent({
  props: defaultHeaderProps,
  inheritAttrs: false,
  setup(props, { slots, emit }) {
    const router = useRouter()
    const context = useRouteContext()

    const { width } = useWindowSize()

    const baseClassName = context.getPrefixCls({
      suffixCls: 'global-header',
      isPor: true
    })
    const className = computed(() => {
      return {
        [`${baseClassName}`]: true,
        [`${props.theme}`]: true,
        ['wide']: props.layout === 'wide'
      }
    })

    const handleChangeKeys = (type: string) => {
      if (router.currentRoute) {
        const matched = router.currentRoute.value.matched.concat()
        if (props.onSelect && type === 'select')
          props.onSelect(matched.filter((r) => r.name !== 'index').map((r) => r.path))
        if (props.onOpenKeys && type === 'openKeys')
          props.onOpenKeys?.(
            matched.filter((r) => r.path !== router.currentRoute.value.path).map((r) => r.path)
          )
      }
    }

    const onCollapse = () => {
      emit('collapse', !props.collapsed)
    }

    const collapsedButtonRender = computed(() => props.collapsedButtonRender || defaultRenderCollapsedButton)

    const contentRender = () => (
      <>
        {props.layout === 'mix' && !props.isMobile && props.splitMenus ? (
          <div style={{ flex: 1 }} class={`${baseClassName}-menu`}>
            <BaseMenu
              theme={props.theme}
              mode={props.mode}
              collapsed={props.collapsed}
              menuData={props.menuData}
              openKeys={context.openKeys}
              selectedKeys={context.selectedKeys}
              iconfontUrl={props.iconfontUrl}
              menuItemRender={props.menuItemRender}
              subMenuItemRender={props.subMenuItemRender}
              class={{ 'top-nav-menu': props.mode === 'horizontal' }}
              {...{
                'onUpdate:openKeys': () => handleChangeKeys('openKeys'),
                'onUpdate:selectedKeys': () => handleChangeKeys('select')
              }}
            />
          </div>
        ) : (
          <div style={{ flex: 1 }}>{slots.default?.()}</div>
        )}
        {props.layout !== 'simple' && (
          <RightContent
            extraRightDropdownRender={props.extraRightDropdownRender}
            rightContentRender={props.rightContentRender}
            {...props}
          />
        )}
      </>
    )

    return () => {
      const logoDom = (
        <span class={[ `${baseClassName}-logo`, props.isMobile ? 'mobile' : '' ]} key="logo">
          <a>{defaultRenderLogo(props.logo || Logo)}</a>
        </span>)
      return (
        <div
          style={{
            height: '100%',
            ...(props.isMobile || props.layout !== 'mix' ? {} : { padding: 0 })
          }}
          class={className.value}
        >
          {props.isMobile ? (
            <>
              {renderLogo(props.menuHeaderRender, logoDom)}
              {props.isMobile && unref(collapsedButtonRender) && (
                <span class={`${baseClassName}-collapsed-button`} onClick={onCollapse}>
              {unref(collapsedButtonRender)(props.collapsed)}
            </span>
              )}
              {contentRender()}
            </>
          ) : props.layout === 'mix' || props.layout === 'wide' ? (
            <>
              <div
                style={{
                  width: props.layout === 'wide' ? `${props.wideWidth}px` : undefined,
                  padding: props.layout === 'wide' && width.value < props.wideWidth ? '0 50px' : undefined
                }}
                class={`${baseClassName}-main`}
              >
                <div class={`${baseClassName}-main-left`}>
                  <LogoContent {...props} renderKey="headerTitleRender" />
                </div>
                {contentRender()}
              </div>
            </>
          ) : (
            contentRender()
          )}
        </div>
      )
    }
  }
})
