import type { Ref, WatchSource } from 'vue'
import { ref, watch, computed, isRef } from 'vue'
import { useDebounceFn } from '@vueuse/core'
import { useState } from '@gx-design-vue/pro-hooks'
import { cloneDeep } from 'lodash-unified'

function useRequest<R, P = any>(
  service: (opt: any) => Promise<ResponseResult<R>>,
  options?: {
    params?: any;
    manual?: boolean;
    ready?: Ref<boolean>;
    onBefore?: () => void;
    debounceInterval?: number;
    refreshDeps?: WatchSource[];
    refreshDepsAction?: () => Promise<any>;
  }
) {
  const ready = computed(() => {
    return !isRef(options?.ready?.value) || options?.ready?.value === undefined ? true : options?.ready?.value
  })

  const [ loading, setLoading ] = useState(false)
  const data = ref<R>()
  const params = ref<P>()

  const query = async (opt: Recordable = {}) => {
    setLoading(true)
    params.value = { ...opt, ...(options?.params || {}) } as P
    const response: ResponseResult<R> = await service(params.value)
    if (response) {
      data.value = cloneDeep(response.data)
    }
    setLoading(false)
  }

  const run = useDebounceFn(query, options.debounceInterval)

  const refresh = () => run()

  if (!options.manual) {
    params.value = { ...(options?.params || {}) } as P
    run()
  }

  if (options.refreshDeps?.length) {
    watch(options.refreshDeps, () => {
      if (options.refreshDepsAction) options?.refreshDepsAction()
      else run()
    }, {
      deep: true
    })
  }

  watch(() => ready.value, (val) => {
    if (val) run()
  }, {
    flush: 'sync'
  })

  return {
    run,
    refresh,
    data,
    params,
    loading
  }
}

export default useRequest
