import type { UserConfig, ConfigEnv, ProxyOptions } from 'vite'
import { loadEnv } from 'vite'
import { resolve } from 'path'
import dayjs from 'dayjs'

import { configManualChunk } from './build/optimizer'
import { createVitePlugins } from './build/plugin'

import pkg from './package.json'

const { dependencies, devDependencies, name, version } = pkg

const __APP_INFO__ = {
  pkg: { dependencies, devDependencies, name, version },
  lastBuildTime: dayjs().format('YYYY-MM-DD HH:mm:ss')
}

function pathResolve(dir: string) {
  return resolve(process.cwd(), '.', dir)
}

function createProxy(prefix) {
  const ret: RecordType = {}
  const proxy: ProxyOptions = {
    target: 'https://127.0.0.1',
    changeOrigin: true,
    ws: true,
    rewrite: (path) => path.replace(new RegExp(`^${prefix}`), `${prefix}`)
  }
  ret[prefix] = proxy
  return ret
}

function wrapperEnv(envConf: RecordType): ViteEnv {
  const ret: any = {}

  for (const envName of Object.keys(envConf)) {
    let realName = envConf[envName].replace(/\\n/g, '\n')
    realName = realName === 'true' ? true : realName === 'false' ? false : realName

    ret[envName] = realName
    process.env[envName] = realName
  }
  return ret
}

export default ({ mode, command }: ConfigEnv): UserConfig => {
  const root = process.cwd()

  const env = loadEnv(mode, root)

  // The boolean type read by loadEnv is a string. This function can be converted to boolean type
  const viteEnv = wrapperEnv(env)

  const { VITE_BASE_URL } = viteEnv

  const isBuild = command === 'build'

  return {
    base: './',
    resolve: {
      alias: [
        {
          find: '@',
          replacement: pathResolve('src') + '/'
        },
        {
          find: '@gx-vuex',
          replacement: pathResolve('src/store') + '/index.ts'
        },
        {
          find: '@gx-mobile/hooks',
          replacement: pathResolve('src/hooks')
        },
        { find: /^~/, replacement: '' }
      ]
    },
    server: {
      host: true,
      port: 9260,
      proxy: createProxy(VITE_BASE_URL)
    },
    plugins: createVitePlugins(viteEnv, isBuild),
    define: {
      __INTLIFY_PROD_DEVTOOLS__: false,
      __APP_INFO__: JSON.stringify(__APP_INFO__)
    },
    build: {
      assetsDir: 'assets',
      target: 'es2015',
      cssTarget: 'chrome80',
      reportCompressedSize: false,
      chunkSizeWarningLimit: 2500,
      commonjsOptions: {
        ignoreTryCatch: false
      },
      rollupOptions: {
        output: {
          manualChunks: configManualChunk
        }
      }
    },
    css: {
      modules: {
        generateScopedName: 'gx-mobile-[local]-[hash:base64:5]'
      },
      preprocessorOptions: {
        less: {
          javascriptEnabled: true
        }
      }
    }
  }
}
